import logging

from django.contrib.admin.views.decorators import staff_member_required
from django.shortcuts import render, redirect

from . import lib


logger = logging.getLogger(__name__)


@staff_member_required
def index(request):
    """
    Render page with some way of firing actions.

    """
    actions = lib.get_actions()

    return render(
        request,
        'django_actions/index.html',
        {
            'actions': actions
        }
    )


@staff_member_required
def do(request, action_name_slugified):
    """
    Perform specified action and then redirect to index page.

    """
    try:
        action = lib.get_action(action_name_slugified)
        logger.info('Running action "{}"...'.format(action.name))
        inputs = [request.POST[input_] for input_ in action.inputs]
        action.do(*inputs)
        logger.info('Finished running action "{}"'.format(action.name))
    except lib.ActionNotFound:
        logger.error('Action not found for slugified name "{}"'.format(action_name_slugified))

    return redirect('django_actions_index')



from django.conf.urls import patterns, url
from .views import index, do

urlpatterns = patterns(
    '',
    url(r'do/(?P<action_name_slugified>[\w-]+)/$', do, name='do_action'),
    url(r'$', index, name='django_actions_index'),
)



import importlib

from django.conf import settings
from django.utils.text import slugify

from funcy import memoize, decorator


class Action(object):
    """
    """
    def __init__(self, name, function, inputs):
        self.name = unicode(name)
        self.name_slugified = slugify(self.name)
        self.function = function
        self.inputs = inputs

    def do(self, *args):
        self.function(*args)


@decorator
def action(call, name, *inputs):
    """
    """
    # import debug
    return Action(name, call, inputs)


class ActionNotFound(Exception):
    """
    """
    pass


def import_from_actions_path():
        module = importlib.import_module(settings.ACTIONS_PATH)
        module_attributes = (getattr(module, attr_) for attr_ in dir(module))

        for attr in module_attributes:

            try:
                candidate = attr()
            except TypeError:
                continue

            if isinstance(candidate, Action):
                yield candidate

        # function = getattr(module)

        # if function:
        #     yield (function_name, function)


@memoize
def get_actions():
    return [a for a in import_from_actions_path()]


@memoize
def get_actions_by_name_slugified():
    return {act.name_slugified: act for act in get_actions()}


def get_action(name):
    actions = get_actions_by_name_slugified()
    try:
        return actions[name]
    except KeyError:
        raise ActionNotFound
